<?php declare(strict_types=1);

namespace App\Entity;

use App\Repository\FoodOrderRepository;
use DateTimeImmutable;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\IdGenerator\UlidGenerator;
use Symfony\Bridge\Doctrine\Types\UlidType;
use Symfony\Component\Uid\Ulid;

#[ORM\Entity(repositoryClass: FoodOrderRepository::class)]
class FoodOrder
{
    #[ORM\Id]
    #[ORM\GeneratedValue(strategy: 'CUSTOM')]
    #[ORM\Column(type: UlidType::NAME, unique: true)]
    #[ORM\CustomIdGenerator(class: UlidGenerator::class)]
    private Ulid $id;

    #[ORM\Column(nullable: true)]
    private DateTimeImmutable|null $closedAt = null;

    #[ORM\Column(length: 30)]
    private string|null $startedBy = null;

    #[ORM\Column]
    private DateTimeImmutable $startedAt;

    #[ORM\OneToMany(
        mappedBy: 'foodOrder',
        targetEntity: OrderItem::class,
        orphanRemoval: true,
    )]
    private Collection $orderItems;

    #[ORM\ManyToOne]
    #[ORM\JoinColumn(nullable: false)]
    private Vendor|null $vendor = null;

    public function __construct()
    {
        $this->id = new Ulid;
        $this->orderItems = new ArrayCollection;
        $this->startedAt = new DateTimeImmutable;
    }

    public function getId(): Ulid
    {
        return $this->id;
    }

    public function getClosedAt(): DateTimeImmutable|null
    {
        return $this->closedAt;
    }

    public function setClosedAt(
        DateTimeImmutable $closedAt = new DateTimeImmutable,
    ): static {
        $this->closedAt = $closedAt;
        return $this;
    }

    public function getStartedBy(): string|null
    {
        return $this->startedBy;
    }

    public function setStartedBy(string $startedBy): static
    {
        $this->startedBy = $startedBy;
        return $this;
    }

    public function getStartedAt(): DateTimeImmutable
    {
        return $this->startedAt;
    }

    public function setStartedAt(DateTimeImmutable $startedAt): static
    {
        $this->startedAt = $startedAt;
        return $this;
    }

    /**
     * @return Collection<int, OrderItem>
     */
    public function getOrderItems(): Collection
    {
        return $this->orderItems;
    }

    public function addOrderItem(OrderItem $orderItem): static
    {
        if (! $this->orderItems->contains($orderItem)) {
            $this->orderItems->add($orderItem);
            $orderItem->setFoodOrder($this);
        }

        return $this;
    }

    public function removeOrderItem(OrderItem $orderItem): static
    {
        // set the owning side to null (unless already changed)
        if (
            $this->orderItems->removeElement($orderItem)
            && $orderItem->getFoodOrder() === $this
        ) {
            $orderItem->setFoodOrder(null);
        }

        return $this;
    }

    public function getVendor(): Vendor|null
    {
        return $this->vendor;
    }

    public function setVendor(Vendor|null $vendor): static
    {
        $this->vendor = $vendor;
        return $this;
    }

    public function groupedOrderItems(): Collection
    {
        return $this->getOrderItems();
    }
}
